package geog;

import java.util.NoSuchElementException;
import java.util.StringTokenizer;

/**
 * A Segment of a Street.
 *
 * @author CS159 Professors, James Madison University
 * @version 1
 */
public class Segment {
    private double length;
    private Location highLocation, lowLocation;
    private int highNumber, lowNumber;
    private String id, type;
    
    /**
     * Default Constructor.
     */
    public Segment() {
    }

    /**
     * Does this Segment contain the given house number?.
     *
     * @param number The house number of interest
     * @return true if yes; false otherwise
     */
    public boolean contains(int number) {
        return number >= lowNumber && number <= highNumber;
    }

    /**
     * Geocode (using linear interpolation) a given house number.
     *
     * @param number The house number of interest
     * @return null if not found; the linear interpolation otherwise
     */
    public OnSegmentLocation interpolate(int number) {
        double latitude, longitude, multiplier;
        OnSegmentLocation result;

        multiplier = 0.0;

        result = null;
        if (contains(number)) {
            // Note: The subtractions involve int values so will be 
            // exactly 0 or 1, both of which can be represented exactly
            // as double values.
            multiplier = (double) (number - lowNumber)
                    / (double) (highNumber - lowNumber);

            longitude = (1.0 - multiplier) * lowLocation.getLongitude()
                    + multiplier * highLocation.getLongitude();

            latitude = (1.0 - multiplier) * lowLocation.getLatitude()
                    + multiplier * highLocation.getLatitude();

            result = new OnSegmentLocation(id, longitude, latitude);
        }

        return result;
    }

    /**
     * Parse the given tab-separated-value (TSV) representation of a Segment and
     * populate the attributes of this Segment object.
     *
     * @param line The String representation
     * @return The ID
     * @throws NoSuchElementException if the String contains the wrong number of
     * fields
     * @throws NumberFormatException if a numeric field is formatted incorrectly
     */
    public String fromTSV(String line)
            throws NoSuchElementException, NumberFormatException {
        double lat, lon;
        StringTokenizer st;

        st = new StringTokenizer(line, "\t");

        id = st.nextToken();

        lon = Double.parseDouble(st.nextToken());
        lat = Double.parseDouble(st.nextToken());
        lowLocation = new Location(lon, lat);

        lon = Double.parseDouble(st.nextToken());
        lat = Double.parseDouble(st.nextToken());
        highLocation = new Location(lon, lat);

        length = Double.parseDouble(st.nextToken());

        type = st.nextToken();

        lowNumber = Integer.parseInt(st.nextToken());
        highNumber = Integer.parseInt(st.nextToken());

        return id;
    }
    
    /**
     * Get the length of this Segment.
     * 
     * @return The length
     */
    public double getLength() {
        return length;
    }
    
    /**
     * Get the type of this Segment.
     * 
     * @return The type
     */
    public String getType() {
        return type;
    }
}
